<?php
/* =========================================================================

PHP Source File -- Created with SAPIEN Technologies PrimalScript 3.1

NAME: class.validator.php

AUTHOR: Charles E Finkenbiner, Creative Electronic Formulas, Inc.
DATE  : 09/24/2014

COMMENT: Generic PHP validator, along the lines of 'JavaScript Form Validation' from
         'http://www.javascript-coder.com/html-form/javascript-form-validation.phtml'
         Some functions were taken and modified to fit my needs from:
         Wixel/GUMP - 'https://github.com/Wixel/GUMP'
         Validation Class article - 'http://www.phpro.org/classes/Validation-Class.html'

         For any feature requests/comments or bugs found please send an e-mail message to
         'http://www.cef-inc.com/contact.php?type=comment'.  If reporting a bug please
         provide as much detail as possible, even better would be to provide a sample page
         that shows the problem, I will provide an upload link when needed

VERSION: 1.0 inital code written
         1.1 fixed problem in trimSource() when an array is passed in
         1.2 added rules 'equal', 'greater_than_equal' and 'less_than_equal'
============================================================================ */
class Validator
{
 // for storing form errors
 public $errors = array();

 // for storing SQL errors
 public $sql_errors = array();

 // for storing validation rules
 protected $rules = array();

 // for storing custom validation functions
 protected $custom = array();

 // for storing sanitized form data
 public $sanitized = array();

 // for storing form data from source
 public $source = array();

 public function addError($field, $errmsg)
 {
  $this->errors[$field][] = $errmsg;
 }

 public function addRule($field, $rule, $errmsg)
 {
  $this->rules[$field][] = array('rule' => $rule, 'errmsg' => $errmsg);
  return $this;
 }

 public function addSQLError($field, $errmsg)
 {
  $this->sql_errors[$field][] = $errmsg;
 }

 public function addSource($source)
 {
  $this->source = $source;
  return $this;
 }

 public function addValidation($func, $field, $errmsg, $parms)
 {
  $this->custom[$field][] = array('function' => $func, 'errmsg' => $errmsg, 'parms' => $parms);
  return $this;
 }

 // $value = String value in HTML field passed to function
 protected function getLength($value)
 {
  if(function_exists('mb_strlen'))
  {
   return mb_strlen($value);
  }
  else
  {
   return strlen($value);
  }
 }

// Run the validation process, return true or false
 public function isFormValid()
 {
  foreach($this->rules as $field => $name)
  {
   foreach($name as $key)
   {
    $rule = explode('=', trim($key['rule']));
    switch(strtolower(trim($rule[0])))
    {
     case 'alnum':
     case 'alphanumeric':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z0-9])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'alnum_d':
     case 'alphanumeric_dash':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z0-9_-])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'alnum_s':
     case 'alphanumeric_space':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z0-9\s])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'alpha':
     case 'alphabetic':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'alpha_d':
     case 'alphabetic_dash':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z_-])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'alpha_s':
     case 'alphabetic_space':
      if(!empty($this->source[$field]) && !preg_match('/^([a-z\s])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'crcard':
      if(!empty($this->source[$field]) && !$this->validateCC($this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'check':
     case 'checked':
      if(!isset($this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'date_f':
      if(!empty($this->source[$field]) && !preg_match('/^([0-9\.\/\-])+$/i', $this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'email':
      if(!empty($this->source[$field]) && !filter_var($this->source[$field], FILTER_VALIDATE_EMAIL))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $email = preg_replace('((?:\n|\r|\t|%0A|%0D|%08|%09)+)i', '', $this->source[$field]);
      $this->source[$field] = filter_var((string)$email, FILTER_SANITIZE_EMAIL);
      break;
     case 'eq':
     case 'equal':
      if(!empty($this->source[$field]) && (filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) != trim((float)$rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((float)$this->source[$field], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_THOUSAND);
      break;
     case 'eqelmnt':
      if(!empty($this->source[$field]) && ($this->source[$field] !== trim($this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'exlen':
     case 'exact_length':
      if(!empty($this->source[$field]) && !$this->validateExlen($this->source[$field], trim($rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'geelmnt':
      if(!empty($this->source[$field]) && !((float)$this->source[$field] >= trim((float)$this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'gtelmnt':
      if(!empty($this->source[$field]) && !((float)$this->source[$field] > trim((float)$this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'gt':
     case 'greater_than':
      if(!empty($this->source[$field]) && (filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) <= trim((float)$rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((float)$this->source[$field], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_THOUSAND);
      break;
     case 'gte':
     case 'greater_than_equal':
      if(!empty($this->source[$field]) && (filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) < trim((float)$rule[1]) || filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) === trim((float)$rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((float)$this->source[$field], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_THOUSAND);
      break;
     case 'int':
     case 'integer':
      if(!empty($this->source[$field]) && !filter_var((int)$this->source[$field], FILTER_VALIDATE_INT))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((int)$this->source[$field], FILTER_SANITIZE_NUMBER_INT);
      break;
     case 'ip':
      if(!empty($this->source[$field]) && filter_var($this->source[$field], FILTER_VALIDATE_IP) === false)
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'ipv4':
      if(!empty($this->source[$field]) && filter_var($this->source[$field], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) === false)
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'ipv6':
      if(!empty($this->source[$field]) && filter_var($this->source[$field], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) === false)
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'lcase':
     case 'lowercase':
      if(!empty($this->source[$field]))
      {
       if(function_exists('mb_strtolower'))
       {
        $this->source[$field] = mb_strtolower($this->source[$field], 'UTF-8');
       }
       else
       {
        $this->source[$field] = strtolower($this->source[$field]);
       }
      }
      break;
     case 'leelmnt':
      if(!empty($this->source[$field]) && !((float)$this->source[$field] <= trim((float)$this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'ltelmnt':
      if(!empty($this->source[$field]) && !((float)$this->source[$field] < trim((float)$this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'lt':
     case 'less_than':
      if(!empty($this->source[$field]) && (filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) >= trim((float)$rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((float)$this->source[$field], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_THOUSAND);
      break;
     case 'lte':
     case 'less_than_equal':
      if(!empty($this->source[$field]) && (filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) > trim((float)$rule[1]) || filter_var((float)$this->source[$field], FILTER_VALIDATE_FLOAT, FILTER_FLAG_ALLOW_THOUSAND) === trim((float)$rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      $this->source[$field] = filter_var((float)$this->source[$field], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_THOUSAND);
      break;
     case 'maxlen':
     case 'max_length':
      if(!empty($this->source[$field]) && !$this->validateMaxlen($this->source[$field], trim($rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'minlen':
     case 'min_length':
      if(!empty($this->source[$field]) && !$this->validateMinlen($this->source[$field], trim($rule[1])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'neelmnt':
      if(!empty($this->source[$field]) && ($this->source[$field] === trim($this->source[$rule[1]])))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'nochk':
     case 'nocheck':
      if(isset($this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'nosel':
     case 'noselect':
      if(isset($this->source[$field]) && $this->source[$field] === trim($rule[1]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'padb':
     case 'padboth':
      if(isset($this->source[$field]))
      {
       $this->source[$field] = str_pad($this->source[$field], trim($rule[1]), trim($key['errmsg']), STR_PAD_BOTH);
      }
      break;
     case 'padl':
     case 'padleft':
      if(isset($this->source[$field]))
      {
       $this->source[$field] = str_pad($this->source[$field], trim($rule[1]), trim($key['errmsg']), STR_PAD_LEFT);
      }
      break;
     case 'padr':
     case 'padright':
      if(isset($this->source[$field]))
      {
       $this->source[$field] = str_pad($this->source[$field], trim($rule[1]), trim($key['errmsg']), STR_PAD_RIGHT);
      }
      break;
     case 'req':
     case 'required':
      if(!isset($this->source[$field]) || $this->source[$field] === '')
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'selrad':
     case 'select_radio':
      if(!isset($this->source[$field]))
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     case 'ucase':
     case 'uppercase':
      if(!empty($this->source[$field]))
      {
       if(function_exists('mb_strtoupper'))
       {
        $this->source[$field] = mb_strtoupper($this->source[$field], 'UTF-8');
       }
       else
       {
        $this->source[$field] = strtoupper($this->source[$field]);
       }
      }
      break;
     case 'url':
      if(!empty($this->source[$field]) && filter_var($this->source[$field], FILTER_VALIDATE_URL) === false)
      {
       $this->addError($field, $key['errmsg']);
       break;
      }
      break;
     default:
      $this->addError($field, "There is no validation rule \"$rule[0]\" for this field");
      break;
    }
   }
  }
  if(sizeof($this->custom) > 0)
  {
   foreach($this->custom as $field => $subkey)
   {
    foreach($subkey as $name)
    {
     if(call_user_func($name['function'], $name['parms']) === false)
     {
      $this->addError($field, $name['errmsg']);
     }
    }
   }
  }
  if(sizeof($this->errors) === 0)
  {
   $this->sanitized = $this->sanitize($this->source);
   return true;
  }
  else
  {
   return false;
  }
 }

 // Parameter of 'false' will print a 1 line warning message, errors can be displayed in individual <span> elements
 // No parameter will default to 'true' and will print all error messages, grouped by element name
 public function outputErrors($detail = true)
 {
  if(!$detail && sizeof($this->errors) > 0)
  {
   echo('<h4 class="error">ERROR: There is a problem with your data, please review!</h4>' . PHP_EOL);
  }
  else
  {
   foreach($this->errors as $name => $msgs)
   {
    // If you wish to show the HTML element name with the error messages, then
    // un-comment the line below and comment the line below that one (exchange // between lines)
    // echo('<h4 class="error">' . $name . ': ' . implode('<br>', $msgs) . '</h4>' . PHP_EOL);
    echo('<h4 class="error">' . implode('<br>', $msgs) . '</h4>' . PHP_EOL);
   }
  }
 }

 public function outputSQLErrors()
 {
  echo('<h4 class="error">' . implode('<br>', $this->sql_errors['sql']) . '</h4>' . PHP_EOL);
 }

 // Returns the sanitized array
 public function returnSource()
 {
  return $this->sanitized;
 }

 // Will sanitize all items in the source ($input) array
 public function sanitize(array $input, $fields = null, $utf8_encode = true)
 {
  if(is_null($fields))
  {
   $fields = array_keys($input);
  }
  foreach($fields as $field)
  {
   if(!isset($input[$field]))
   {
    continue;
   }
   else
   {
    $value = $input[$field];
    if(is_string($value))
    {
     $value = $this->sanitizeItem($value, $remove_nl=true);
     if(function_exists('iconv') && function_exists('mb_detect_encoding') && $utf8_encode)
     {
      $current_encoding = mb_detect_encoding($value);
      if($current_encoding != 'UTF-8' && $current_encoding != 'UTF-16')
      {
       $value = iconv($current_encoding, 'UTF-8', $value);
      }
     }
     $value = filter_var($value, FILTER_SANITIZE_STRING);
    }
    $this->sanitized[$field] = $value;
   }
  }
  return $this->sanitized;
 }

 protected function sanitizeItem($value, $remove_nl=true)
 {
  $value = trim($value);
  $value = $this->stripAllSlashes($value);
  if($remove_nl)
  {
   $injections = array('/(\n+)/i', '/(\r+)/i', '/(\t+)/i', '/(%0A+)/i', '/(%0D+)/i', '/(%08+)/i', '/(%09+)/i');
   $value = preg_replace($injections, '', $value);
  }
  return $value;
 }

 protected function stripAllSlashes($value)
 {
  if(get_magic_quotes_gpc())
  {
   $value = stripAllSlashes($value);
  }
  return $value;
 }

 // Trim whitespace from both ends in the source array
 public function trimSource()
 {
  foreach($this->source as $field => $value)
  {
   if(is_array($this->source[$field]))
   {
    $this->source[$field] = array_map('trim', $value);
   }
   else
   {
    $this->source[$field] = trim($value);
   }
  }
 }

 // $value = String value in HTML field
 protected function validateCC($value)
 {
  $number = preg_replace('/\D/', '', $value);
  $number_length = $this->getLength($number);
  $parity = $number_length % 2;
  $total = 0;
  for($i = 0; $i < $number_length; $i++)
  {
   $digit = $number[$i];
   if($i % 2 == $parity)
   {
    $digit *= 2;
    if($digit > 9)
    {
     $digit -= 9;
    }
   }
   $total += $digit;
  }
  if($total % 10 == 0)
  {
   return true;
  }
  return false;
 }

 // $value1 = String value in HTML field
 // $value2 = String value of $rule[1]
 protected function validateExlen($value1, $value2)
 {
  $number_length = $this->getLength($value1);
  if((int)$number_length === (int)$value2)
  {
   return true;
  }
  return false;
 }

 // $value1 = String value in HTML field
 // $value2 = String value of $rule[1]
 protected function validateMaxlen($value1, $value2)
 {
  $number_length = $this->getLength($value1);
  if((int)$number_length <= (int)$value2)
  {
   return true;
  }
  return false;
 }

 // $value1 = String value in HTML field
 // $value2 = String value of $rule[1]
 protected function validateMinlen($value1, $value2)
 {
  $number_length = $this->getLength($value1);
  if((int)$number_length >= (int)$value2)
  {
   return true;
  }
  return false;
 }

} // end of class
?>
